@@ -524,7 +524,7 @@ to_str(Val) ->
 
 init([Frame, Name, {OpaVal,Tiled}=State]) ->
     #{bg:=BG, text:=_FG} = Cols = wings_frame:get_colors(),
-    Panel = wxPanel:new(Frame, [{style, ?wxNO_BORDER}, {size,{200,300}}]),
+    Panel = wxPanel:new(Frame, [{style, wings_frame:get_border()}, {size,{200,300}}]),
     wxPanel:setFont(Panel, ?GET(system_font_wx)),
     wxWindow:setBackgroundColour(Panel, BG),
     Main = wxBoxSizer:new(?wxVERTICAL),
@@ -536,13 +536,17 @@ init([Frame, Name, {OpaVal,Tiled}=State]) ->
     setup_image_list(ImgLst),
     BAct = wxToggleButton:new(Panel, ?wxID_ANY, snap_label(activate)),
     wxWindow:setToolTip(BAct, wxToolTip:new(snap_tooltip(activate))),
-    CtrlBox = wxPanel:new(Panel, [{style, ?wxNO_BORDER}]),
+    CtrlBox = wxPanel:new(Panel),
     wxWindow:setBackgroundColour(CtrlBox, BG),
     %% layout settings for the first level controls
     Szr1 = wxBoxSizer:new(?wxVERTICAL),
-    wxSizer:add(Szr1, ImgLst, [{flag, ?wxEXPAND}]),
-    wxSizer:add(Szr1, BAct, [{flag, ?wxEXPAND}]),
-    wxSizer:add(Szr1, CtrlBox, [{proportion, 1}, {border, 2}, {flag, ?wxEXPAND}]),
+    ExpandBorder = [{border, 3}, {flag, ?wxEXPAND bor  ?wxLEFT bor ?wxRIGHT}],
+    wxSizer:addSpacer(Szr1, 3),
+    wxSizer:add(Szr1, ImgLst, [{border, 7}, {flag, ?wxEXPAND bor ?wxLEFT bor ?wxRIGHT}]),
+    wxSizer:addSpacer(Szr1, 3),
+    wxSizer:add(Szr1, BAct,   [{border, 6}, {flag, ?wxEXPAND bor ?wxLEFT bor ?wxRIGHT}]),
+    wxSizer:addSpacer(Szr1, 3),
+    wxSizer:add(Szr1, CtrlBox, [{proportion, 1}|ExpandBorder]),
 
     %% second level (Panel content): Snap Image, Opacity, Rotation
     BSnap = wxButton:new(CtrlBox, ?wxID_ANY, [{label,snap_label(snap)}]),
@@ -556,20 +560,22 @@ init([Frame, Name, {OpaVal,Tiled}=State]) ->
     RotLbl = wxStaticText:new(CtrlBox, ?wxID_ANY, append_value(rotate,0.0)),
     Rot = wxSlider:new(CtrlBox, ?wxID_ANY, 0, -360, 360),
     wxWindow:setToolTip(Rot, wxToolTip:new(snap_tooltip(rotate))),
-    ModLbx = wxListBox:new(CtrlBox, ?wxID_ANY, [{style, ?wxLB_SINGLE}, {choices, mod_choices()}]),
+    ModLbx = wxListBox:new(CtrlBox, ?wxID_ANY,
+                           [{style, ?wxLB_SINGLE bor ?wxBORDER_NONE},
+                            {choices, mod_choices()}]),
     wxWindow:setBackgroundColour(ModLbx, BG),
     %% layout settings for the second level controls
     Szr2 = wxBoxSizer:new(?wxVERTICAL),
-    wxSizer:add(Szr2, BSnap, [{proportion, 0}, {border, 2}, {flag, ?wxEXPAND}]),
+    wxSizer:add(Szr2, BSnap, [{proportion, 0}|ExpandBorder]),
     wxSizer:addSpacer(Szr2, 3),
-    wxSizer:add(Szr2, TileChk, [{proportion, 0}, {border, 2}, {flag, ?wxEXPAND}]),
+    wxSizer:add(Szr2, TileChk, [{proportion, 0}|ExpandBorder]),
     wxSizer:addSpacer(Szr2, 3),
-    wxSizer:add(Szr2, OpaLbl, [{proportion, 0}, {border, 2}, {flag, ?wxEXPAND}]),
-    wxSizer:add(Szr2, Opa, [{proportion, 0}, {border, 2}, {flag, ?wxEXPAND}]),
-    wxSizer:add(Szr2, RotLbl, [{proportion, 0}, {border, 2}, {flag, ?wxEXPAND}]),
-    wxSizer:add(Szr2, Rot, [{proportion, 0}, {border, 2}, {flag, ?wxEXPAND}]),
+    wxSizer:add(Szr2, OpaLbl, [{proportion, 0}|ExpandBorder]),
+    wxSizer:add(Szr2, Opa, [{proportion, 0}|ExpandBorder]),
+    wxSizer:add(Szr2, RotLbl, [{proportion, 0}|ExpandBorder]),
+    wxSizer:add(Szr2, Rot, [{proportion, 0}|ExpandBorder]),
     wxSizer:addSpacer(Szr2, 3),
-    wxSizer:add(Szr2, ModLbx, [{proportion, 1}, {border, 2}, {flag, ?wxEXPAND}]),
+    wxSizer:add(Szr2, ModLbx, [{proportion, 1}|ExpandBorder]),
     wxPanel:setSizer(CtrlBox, Szr2),
 
     wxSizer:add(Main, Szr1, [{proportion, 1}, {flag, ?wxEXPAND}]),
