@@ -14,7 +14,7 @@
 
 -module(wpc_sweep_extrude).
 -export([init/0,menu/2,command/2]).
--include("wings.hrl").
+-include_lib("wings/src/wings.hrl").
 
 init() ->
     wings_pref:delete_value(sweep_mode),
