@@ -18,11 +18,11 @@
 -export([file_filters/1]).
 
 -include("wings.hrl").
--include("e3d.hrl").
--include("e3d_image.hrl").
+-include_lib("wings/e3d/e3d.hrl").
+-include_lib("wings/e3d/e3d_image.hrl").
 -include_lib("kernel/include/file.hrl").
 
--import(lists, [reverse/1,keymember/3,foreach/2]).
+-import(lists, [foldl/3,foreach/2,keymember/3,reverse/1]).
 -import(filename, [dirname/1]).
 
 -define(WINGS, ".wings").
@@ -165,8 +165,8 @@ menu() ->
      separator,
      {?__(23,"Render"),{render,[]}},
      separator,
-     {?__(24,"Install Plug-In"),install_plugin,
-      ?__(27,"Install a plug-in")},
+     {?__(24,"Install Plug-In or Patch"),install_plugin,
+      ?__(27,"Install a plug-in or a wings patch file")},
      separator,
      {?__(31,"Save Preference Subset..."),save_pref,
       ?__(32,"Save a preference subset from your current settings")},
@@ -231,6 +231,8 @@ command(import_image, _St) ->
     import_image();
 command({import_image,Name}, _) ->
     import_image(Name);
+command({import_files, Fs}, St) ->
+    {save_state, wpa:import(Fs, St)};
 command({export,ndo}, St) ->
     String = case os:type() of
         {win32,_} -> "Export";
@@ -245,12 +247,9 @@ command({export_selected,ndo}, St) ->
     export_ndo(export_selected, String, St);
 command({export,{ndo,Filename}}, St) ->
     do_export_ndo(Filename, St);
-command({export_selected,{ndo,Filename}}, St) ->
-    Shs0 = wings_sel:fold(fun(_, #we{id=Id}=We, A) ->
-				  [{Id,We}|A]
-			  end, [], St),
-    Shs = gb_trees:from_orddict(reverse(Shs0)),
-    do_export_ndo(Filename, St#st{shapes=Shs});
+command({export_selected,{ndo,Filename}}, St0) ->
+    St = delete_unselected(St0),
+    do_export_ndo(Filename, St);
 command(install_plugin, _St) ->
     install_plugin();
 command({install_plugin,Filename}, _St) ->
@@ -437,7 +436,7 @@ save_now(Next, #st{file=Name0}=St) ->
     end,
     file:rename(Name, Backup),
     file:delete(autosave_filename(Name)),
-    case ?SLOW(wings_ff_wings:export(Name, St)) of
+    case ?SLOW(wings_ff_wings:export(Name, false, St)) of
 	ok ->
 	    set_cwd(dirname(Name)),
 	    add_recent(Name),
@@ -473,11 +472,9 @@ save_selected(St) ->
     Cont = fun(Name) -> {file,{save_selected,Name}} end,
     export_filename(Ps, St, Cont).
 
-save_selected(Name, #st{shapes=Shs0,sel=Sel}=St0) ->
-    Shs = [Sh || {Id,_}=Sh <- gb_trees:to_list(Shs0),
-		 keymember(Id, 1, Sel)],
-    St = St0#st{shapes=gb_trees:from_orddict(Shs)},
-    case ?SLOW(wings_ff_wings:export(Name, St)) of
+save_selected(Name, St0) ->
+    St = delete_unselected(St0),
+    case ?SLOW(wings_ff_wings:export(Name, true, St)) of
 	ok -> keep;
 	{error,Reason} -> wings_u:error_msg(Reason)
     end.
@@ -549,7 +546,10 @@ use_autosave_1(#file_info{mtime=SaveTime0}, File, Body) ->
 	{ok,#file_info{mtime=AutoInfo0}} ->
 	    SaveTime = calendar:datetime_to_gregorian_seconds(SaveTime0),
 	    AutoTime = calendar:datetime_to_gregorian_seconds(AutoInfo0),
+            FastStart = wings:is_fast_start(),
 	    if
+                FastStart -> %% Ignore autosave
+                    Body(File);
 		AutoTime > SaveTime ->
 		    Msg = ?__(1,"An autosaved file with a later time stamp exists;"
                               " do you want to load the autosaved file instead?"),
@@ -618,7 +618,7 @@ autosave(#st{file=Name}=St) ->
     View = wings_wm:get_prop(geom, current_view),
     wings_view:set_current(View),
     filelib:ensure_dir(Auto),
-    case ?SLOW(wings_ff_wings:export(Auto, St)) of
+    case ?SLOW(wings_ff_wings:export(Auto, false, St)) of
 	ok ->
 	    wings_u:caption(St#st{saved=auto});
 	{error,Reason} ->
@@ -836,3 +836,7 @@ clean_images(#st{saved=Limit}=St) when is_integer(Limit) ->
 
 clean_new_images(#st{saved=Limit}) when is_integer(Limit) ->
     wings_image:delete_from(Limit).
+
+delete_unselected(St) ->
+    Unselected = wings_sel:unselected_ids(St),
+    foldl(fun wings_obj:delete/2, St, Unselected).
