@@ -656,10 +656,10 @@ draw_normals(#dlo{normals=Ns}, RS) ->
     wings_dl:call(Ns, RS).
 
 edge_width(edge) -> wings_pref:get_value(edge_width);
-edge_width(_) -> 1.
+edge_width(_) -> 1.0.
 
 hard_edge_width(edge) -> wings_pref:get_value(hard_edge_width);
-hard_edge_width(_) -> max(wings_pref:get_value(hard_edge_width) - 1, 1).
+hard_edge_width(_) -> max(wings_pref:get_value(hard_edge_width) - 1, 1.0).
 
 ground_and_axes(#view{yon=Yon0} = View, PM,MM) ->
     Axes = wings_wm:get_prop(show_axes),
@@ -708,7 +708,7 @@ axis_letters(TPM, TMM, {ShowGrid, ShowAxes, Yon0}) ->
 	    gl:matrixMode(?GL_PROJECTION),
 	    gl:loadIdentity(),
 	    {_,_,W,H} = ViewPort,
-	    glu:ortho2D(0.0, W, H, 0.0),
+	    glu:ortho2D(0.0, float(W), float(H), 0.0),
 	    gl:matrixMode(?GL_MODELVIEW),
 	    gl:loadIdentity(),
 	    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
@@ -822,8 +822,9 @@ clip_1({O1,D1}=Axis, [{O2,D2}|Lines], {Ow,_}=W) ->
 clip_1(_, [], _W) -> none.
 
 show_letter(X0, Y0, W, Char, {_,_,Vw,Vh}) ->
-    X = trunc((0.5*X0/W+0.5)*(Vw-20) + 10),
-    Y = Vh - trunc((0.5*Y0/W+0.5)*(Vh-16) - 1),
+    PosX = trunc((0.5*X0/W+0.5)*(Vw-20) + 10),
+    X = max(5, min(Vw-20, PosX)),
+    Y = max(30, min(Vh-20, Vh - trunc((0.5*Y0/W+0.5)*(Vh-16) - 1))),
     axis_text(X, Y, Char).
 
 axis_text(X, Y, C) ->
