@@ -24,9 +24,14 @@
 	 pos/2,outer_vertices_ccw/2,reachable/2,
 	 isolated/1,edge_through/3,edge_through/4]).
 
+-export_type([vertex_num/0]).
+
 -include("wings.hrl").
 -import(lists, [member/2,foldl/3,reverse/1,sort/1]).
 
+-type vertex_num() :: non_neg_integer().
+-type edge_num() :: wings_edge:edge_num().
+
 %% from_faces(FaceGbSet, We) -> VertexList
 %%  Convert a set of faces to a list of vertices.
 from_faces(Fs, We) ->
@@ -240,8 +245,13 @@ connect(Face, Vs, #we{} = We0) ->
 	#we{} = We -> We
     end.
 
-%% Connect Va Vb maybe trough several faces, returns {gbset(NewEdges), We}
-connect_cut(VS0, VE0, #we{}=We0) when is_integer(VS0),is_integer(VE0) ->
+%% Connect Va to Vb, maybe trough several faces.
+%% Return {#we{},gbset(Edges).
+
+-spec connect_cut(vertex_num(), vertex_num(), #we{}) ->
+                         {#we{},gb_sets:set(edge_num())}.
+
+connect_cut(VS0, VE0, #we{}=We0) when is_integer(VS0), is_integer(VE0) ->
     CutPlane = calc_planes(VS0,VE0,We0),
     ETree = collect_edges(CutPlane, ordsets:from_list([VS0, VE0]), We0),
     try
@@ -268,7 +278,7 @@ connect_vs([Va, Face|[Vb|_]=Vs], #we{next_id=Edge}=We0, Acc) ->
 	    connect_vs(Vs, We0, [Exist|Acc])
     end;
 connect_vs([_, undefined], We, Acc) ->
-    {gb_sets:from_list(Acc), We}.
+    {We,gb_sets:from_list(Acc)}.
 
 path_length([V1|[V2|_]=Vs], We, Acc) ->
     Dist = e3d_vec:dist(v(V1, We),v(V2, We)),
@@ -614,6 +624,8 @@ try_connect_1(Va, Vb, Face, We0) ->
 %%  and the left side of the new edge will belong to the new face.
 %%
 force_connect(Vstart, Vend, Face, #we{es=Etab0,fs=Ftab0}=We0) ->
+    All = wings_face:vertices_ccw(Face, We0),
+    true = lists:member(Vstart, All) andalso lists:member(Vend, All),
     {NewFace,We1} = wings_we:new_ids(1, We0),
     NewEdge = NewFace,
     NeRec0 = #edge{vs=Vstart,ve=Vend,lf=NewFace,rf=Face},
