@@ -30,7 +30,7 @@
 
 %% Window information.
 -export([top_size/0,
-	 viewport/0, viewport/1,
+	 viewport/0, viewport/1, win_scale/0,
 	 win_size/0, win_size/1,
 	 win_rect/0, win_rect/1,
 	 win_ul/0, win_ul/1, win_ur/1,win_ll/1,win_lr/1,win_z/1
@@ -43,9 +43,6 @@
 %% Setting the information bar message.
 -export([message/1,message/2,message_right/1]).
 
-%% Drag & Drop support.
--export([drag/3,drag/4,allow_drag/1]).
-
 %% Window property mangagement.
 -export([get_props/1,get_prop/1,get_prop/2,lookup_prop/1,lookup_prop/2,
 	 set_win_props/2, set_prop/2,set_prop/3,erase_prop/1,erase_prop/2,
@@ -70,7 +67,8 @@
 	 stk,					%Event handler stack.
 	 links=[],			        %Windows linked to this one.
 	 dd=none, 				%Display data cache
-	 obj                                    %wx window
+	 obj,                                   %wx window
+         scale=1                                %HighDpi scale factor
 	}).
 
 -record(se,					%Stack entry record.
@@ -131,9 +129,10 @@ new(Name, Obj, Op) when element(1, Obj) =:= wx_ref ->
     new_props(Name, [{font,system_font}]),
     Z = highest_z(),
     {W,H} = wxWindow:getClientSize(Obj),
-    %% io:format("~p:~p: ~p ~p~n",[?MODULE, ?LINE, Name, Obj]),
+    Scale = wxWindow:getContentScaleFactor(Obj),
+    %% io:format("~p:~p: ~p ~p ~p~n",[?MODULE, ?LINE, Name, Obj, Scale]),
     use_opengl(Obj) andalso init_opengl(Name, Obj),
-    Win = #win{x=0,y=0,z=Z,w=W,h=H,name=Name,stk=Stk,obj=Obj},
+    Win = #win{x=0,y=0,z=Z,w=W,h=H,name=Name,stk=Stk,obj=Obj, scale=Scale},
     put(wm_windows, gb_trees:insert(Name, Win, get(wm_windows))),
     put(Obj, Name),
     dirty().
@@ -237,7 +236,10 @@ new_resolve_z(highest) ->
 new_resolve_z(Z) when is_integer(Z), Z >= 0-> Z.
 
 delete(Name) ->
-    wings_frame:close(wxwindow(Name)),
+    case wxwindow(Name) of
+        undefined -> ignore;
+        Win -> wings_frame:close(Win)
+    end,
     Windows = delete_windows(Name, get(wm_windows)),
     put(wm_windows, Windows),
     case is_window(grabbed_focus_window()) of
@@ -347,6 +349,7 @@ is_geom() ->
     case this() of
       geom -> true;
       {geom,_} -> true;
+      {plugin, {_, geom}} -> true;
       _ -> false
     end.
 
@@ -496,6 +499,10 @@ cancel_timer(Ref) ->
 viewport() ->
     get(wm_viewport).
 
+win_scale() ->
+    #win{scale=Scale} = get_window_data(this()),
+    Scale.
+
 viewport(Name) ->
     #win{x=X,y=Y0,w=W,h=H} = get_window_data(Name),
     {_,TopH} = get(wm_top_size),
@@ -572,13 +579,30 @@ get_value(Key) ->
     end.
 
 set_value(Key,Val) ->
-    put(Key, Val),
+    case cache_key(Key) of
+        true ->
+            %% (self() =:= whereis(wings)) orelse
+            %%     io:format("Cached value (~p) updated from wrong process~n",[Key]),
+            put(Key, Val);
+        false -> ok
+    end,
     wings_pref:set_value({temp, Key}, Val).
 
 delete_value(Key) ->
     erase(Key),
     wings_pref:delete_value({temp, Key}).
 
+
+cache_key(cursors) -> true;
+cache_key(wings_plugins) -> true;
+cache_key(gl_canvas) -> true;
+cache_key(top_frame) -> true;
+cache_key(light_shaders) -> true;
+cache_key(system_font) -> true;
+cache_key({geom, _}) -> true;  %% Only set by wings_wm?
+cache_key({_, props}) -> true;  %% Only set by wings_wm?
+cache_key(_) -> false.
+
 new_props(Win, Props0) ->
     Props = gb_trees:from_orddict(Props0),
     ?SET({Win, props}, Props).
@@ -714,6 +738,12 @@ dispatch_event(#wx{event=#wxActivate{active=Active}}) ->
 	    dirty(),
             true;
 	false ->
+            case get_focus_window() of
+                {grabbed, Grab} ->
+                    do_dispatch(Grab, lost_focus);
+                _ ->
+                    ignore
+            end,
 	    update_focus(none),
 	    true
     end;
@@ -747,7 +777,8 @@ dispatch_event(#wx{obj=Obj, event=#wxFocus{type=kill_focus, win=New}}) ->
         {{grabbed, OtherWin}, Win} ->
             case OtherWin of
                 dialog_blanket -> ignore;
-                _ -> ?dbg("Grabbed focus lost old?: ~p ~p~n", [OtherWin, Win])
+                _ ->
+                    ?dbg("Grabbed focus lost old?: ~p ~p ~p~n", [OtherWin, Win, New])
             end,
             ok;
         {_OldFocus, _CurrentFocus} ->
@@ -871,7 +902,7 @@ redraw_all() ->
     wings_io:set_cursor(get(wm_cursor)),
     event_loop().
 
-redraw_win({Name, #win{w=W,h=H,obj=Obj}}) ->
+redraw_win({Name, #win{w=W,h=H,obj=Obj,scale=Scale}}) ->
     DoSwap = case use_opengl(Obj) of
 		 true ->
                      case get(current_gl) of
@@ -880,7 +911,7 @@ redraw_win({Name, #win{w=W,h=H,obj=Obj}}) ->
                              wxGLCanvas:setCurrent(Obj),
                              put(current_gl, Obj)
                      end,
-		     gl:viewport(0,0,W,H),
+		     gl:viewport(0,0,round(W*Scale),round(H*Scale)),
 		     gl:clear(?GL_COLOR_BUFFER_BIT bor ?GL_DEPTH_BUFFER_BIT),
 		     true;
 		 false ->
@@ -1124,7 +1155,15 @@ wm_event({message_right,Name,Right0}) ->
 	    dirty()
     end;
 wm_event({callback,Cb}) ->
-    Cb().
+    Cb();
+wm_event({drop, GlobalPos, Drop}) ->
+    case window_below(GlobalPos) of
+        none ->
+            ok;
+        Win ->
+            send(Win, {drop, Drop}),
+            ok
+    end.
 
 %%%
 %%% Finding the active geom window, wings_wm only handles focus of geom windows
@@ -1136,8 +1175,6 @@ find_active() ->
  	Focus -> Focus
     end.
 
-window_below(X,Y) ->
-    window_below({X,Y}).
 window_below(Pos) ->
     Win0 = wx_misc:findWindowAtPoint(Pos),
     case wx:is_null(Win0) of
@@ -1178,120 +1215,6 @@ geom_below(Pos) ->
             find_window(Geoms, Geoms, Win0)
     end.
 
-%%%
-%%% Drag and drop support.
-%%%
--record(drag,
-	{data,					%Drop data.
-	 bstate,				%State of mouse buttons.
-	 redraw,				%Redraw function.
-	 over=none,				%We are over this window.
-	 drop_ok=false				%Drop is OK on this window.
-	}).
-
-allow_drag(false) -> allow_drag_1(arrow);
-allow_drag(true) -> allow_drag_1(pointing_hand).
-
-allow_drag_1(Cursor) ->
-    case get(wm_cursor) of
-	Cursor -> ok;
-	_ ->
-	    put(wm_cursor, Cursor),
-	    wings_io:set_cursor(Cursor)
-    end.
-
-drag(Ev, Rect, DropData) ->
-    Redraw = fun() ->
-		     gl:pushAttrib(?GL_POLYGON_BIT bor ?GL_LINE_BIT),
-		     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-		     gl:lineStipple(2, 2#0101010101010101),
-		     gl:enable(?GL_LINE_STIPPLE),
-		     gl:color3b(0, 0, 0),
-		     {W,H} = wings_wm:win_size(),
-		     gl:rectf(0.5, 0.5, W-1, H-1),
-		     gl:popAttrib()
-	     end,
-    drag(Ev, Rect, Redraw, DropData).
-
-drag(#mousemotion{x=X,y=Y,state=State}, Rect, Redraw, DropData) ->
-    drag_1(X, Y, State, Rect, Redraw, DropData);
-drag(#mousebutton{x=X,y=Y,button=B}, Rect, Redraw, DropData) ->
-    State = 1 bsl (B-1),
-    drag_1(X, Y, State, Rect, Redraw, DropData).
-
-drag_1(X1, Y1, State, {W,H}, Redraw, DropData) ->
-    X = X1 - W div 2,
-    Y = Y1 - H div 2,
-    Drag = #drag{data=DropData,bstate=State,redraw=Redraw},
-    Op = {seq,push,get_drag_event(Drag)},
-    Name = dragger,
-    new(Name, {X,Y,highest}, {W,H}, Op),
-    grab_focus(Name),
-    dirty(),
-    keep.
-
-get_drag_event(Drag) ->
-    {replace,fun(Ev) -> drag_event(Ev, Drag) end}.
-		     
-drag_event(redraw, #drag{redraw=Redraw}) ->
-    wings_io:ortho_setup(),
-    Redraw(),
-    keep;
-drag_event(#mousemotion{x=X0,y=Y0}, #drag{over=Over0}=Drag0) ->
-    {W,H} = wings_wm:win_size(),
-    offset(dragger, X0 - W div 2, Y0 - H div 2),
-    {X,Y} = local2screen({X0, Y0}),
-    hide(dragger),
-    Over = window_below(X, Y),
-    show(dragger),
-    case Over of
-	Over0 -> keep;
-	_ ->
-	    Drag = Drag0#drag{over=Over},
-	    drag_filter(Drag)
-    end;
-drag_event(#mousebutton{button=B,state=?SDL_RELEASED},
-	   #drag{bstate=State,data=DropData,drop_ok=DropOK}) ->
-    if
-	((1 bsl (B-1)) band State) =/= 0 ->
-	    case DropOK of
-		false -> ok;
-		true ->
-		    {X,Y,W,H} = wings_wm:win_rect(dragger),
-		    Ev = {drop,{X + W div 2,Y + H div 2},DropData},
-		    wings_io:putback_event(Ev)
-	    end,
-	    put(wm_cursor, arrow),
-	    delete;
-	true -> keep
-    end;
-drag_event(_, _) -> keep.
-
-drag_filter(#drag{over=none}=Drag) ->
-    stop_cursor(Drag);
-drag_filter(#drag{over=Win,data=DropData}=Drag) ->
-    case lookup_prop(Win, drag_filter) of
-	{value,Fun} when is_function(Fun) ->
-	    case Fun(DropData) of
-		yes ->
-		    message(""),
-		    put(wm_cursor, closed_hand),
-		    get_drag_event(Drag#drag{drop_ok=true});
-		{yes,Message} ->
-		    message(Message),
-		    put(wm_cursor, closed_hand),
-		    get_drag_event(Drag#drag{drop_ok=true});
-		no ->
-		    stop_cursor(Drag)
-	    end;
-	none -> stop_cursor(Drag)
-    end.
-
-stop_cursor(Drag) ->
-    put(wm_cursor, stop),
-    message(""),
-    get_drag_event(Drag#drag{drop_ok=false}).
-
 %%%
 %%% Utility functions.
 %%%
